# -*- coding: utf-8; -*-

from httpolice.citation import RFC, Citation
from httpolice.known.base import KnownDict
from httpolice.structure import FieldName
from httpolice.syntax import (internal, rfc5789, rfc5988, rfc6266, rfc6797,
                              rfc7230, rfc7231, rfc7232, rfc7233, rfc7234,
                              rfc7235, rfc7540, rfc7838)


SINGLE = 1
SINGLE_LIST = 2
MULTI = 3
SET_COOKIE = 4


def deprecated(name):
    return known.get_info(name).get('iana_status') in [u'deprecated',
                                                       u'obsoleted']

def is_bad_for_connection(name):
    return known.get_info(name).get('bad_for_connection')

def is_bad_for_trailer(name):
    return known.get_info(name).get('bad_for_trailer')

def is_for_request(name):
    return known.get_info(name).get('for_request')

def is_for_response(name):
    return known.get_info(name).get('for_response')

def is_precondition(name):
    return known.get_info(name).get('precondition')

def is_proactive_conneg(name):
    return known.get_info(name).get('proactive_conneg')

def is_representation_metadata(name):
    return known.get_info(name).get('representation_metadata')

def rule_for(name):
    return known.get_info(name).get('rule')

def parser_for(name):
    return known.get_info(name).get('parser')


known = KnownDict([
 {'_': FieldName(u'A-IM'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Accept'),
  '_citations': [RFC(7231, section=(5, 3, 2))],
  'for_request': True,
  'for_response': False,
  'iana_status': u'standard',
  'parser': rfc7231.Accept,
  'precondition': False,
  'proactive_conneg': True,
  'rule': MULTI},
 {'_': FieldName(u'Accept-Additions'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Accept-Charset'),
  '_citations': [RFC(7231, section=(5, 3, 3))],
  'for_request': True,
  'for_response': False,
  'iana_status': u'standard',
  'parser': rfc7231.Accept_Charset,
  'precondition': False,
  'proactive_conneg': True,
  'rule': MULTI},
 {'_': FieldName(u'Accept-Datetime'),
  '_citations': [RFC(7089)],
  'iana_status': 'informational'},
 {'_': FieldName(u'Accept-Encoding'),
  '_citations': [RFC(7231, section=(5, 3, 4)), RFC(7694, section=(3,))],
  'for_request': True,
  'for_response': True,         # RFC 7694
  'iana_status': u'standard',
  'parser': rfc7231.Accept_Encoding,
  'precondition': False,
  'proactive_conneg': True,
  'rule': MULTI},
 {'_': FieldName(u'Accept-Features'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Accept-Language'),
  '_citations': [RFC(7231, section=(5, 3, 5))],
  'for_request': True,
  'for_response': False,
  'iana_status': u'standard',
  'parser': rfc7231.Accept_Language,
  'precondition': False,
  'proactive_conneg': True,
  'rule': MULTI},
 {'_': FieldName(u'Accept-Patch'),
  '_citations': [RFC(5789, section=(3, 1))],
  'for_request': False,
  'for_response': True,
  'parser': rfc5789.Accept_Patch,
  'rule': MULTI},
 {'_': FieldName(u'Accept-Ranges'),
  '_citations': [RFC(7233, section=(2, 3))],
  'for_request': False,
  'for_response': True,
  'iana_status': u'standard',
  'parser': rfc7233.Accept_Ranges,
  'rule': SINGLE},
 {'_': FieldName(u'Access-Control'),
  '_citations': [Citation(u'W3C Web Application Formats Working Group',
                          u'http://www.w3.org/2006/appformats/')],
  'iana_status': u'deprecated'},
 {'_': FieldName(u'Access-Control-Allow-Credentials'),
  '_citations': [Citation(u'W3C Cross-Origin Resource Sharing',
                          u'https://www.w3.org/TR/cors/'
                          u'#access-control-allow-credentials-'
                          u'response-header')]},
 {'_': FieldName(u'Access-Control-Allow-Headers'),
  '_citations': [Citation(u'W3C Cross-Origin Resource Sharing',
                          u'https://www.w3.org/TR/cors/'
                          u'#access-control-allow-headers-response-header')]},
 {'_': FieldName(u'Access-Control-Allow-Methods'),
  '_citations': [Citation(u'W3C Cross-Origin Resource Sharing',
                          u'https://www.w3.org/TR/cors/'
                          u'#access-control-allow-methods-response-header')]},
 {'_': FieldName(u'Access-Control-Allow-Origin'),
  '_citations': [Citation(u'W3C Cross-Origin Resource Sharing',
                          u'https://www.w3.org/TR/cors/'
                          u'#access-control-allow-origin-response-header')]},
 {'_': FieldName(u'Access-Control-Max-Age'),
  '_citations': [Citation(u'W3C Cross-Origin Resource Sharing',
                          u'https://www.w3.org/TR/cors/'
                          u'#access-control-max-age-response-header')]},
 {'_': FieldName(u'Access-Control-Request-Method'),
  '_citations': [Citation(u'W3C Cross-Origin Resource Sharing',
                          u'https://www.w3.org/TR/cors/'
                          u'#access-control-request-method-request-header')]},
 {'_': FieldName(u'Access-Control-Request-Headers'),
  '_citations': [Citation(u'W3C Cross-Origin Resource Sharing',
                          u'https://www.w3.org/TR/cors/'
                          u'#access-control-request-headers-request-header')]},
 {'_': FieldName(u'Age'),
  '_citations': [RFC(7234, section=(5, 1))],
  'bad_for_connection': True,
  'bad_for_trailer': True,
  'for_request': False,
  'for_response': True,
  'iana_status': u'standard',
  'parser': rfc7234.Age,
  'rule': SINGLE},
 {'_': FieldName(u'Allow'),
  '_citations': [RFC(7231, section=(7, 4, 1))],
  'for_request': False,
  'for_response': True,
  'iana_status': u'standard',
  'parser': rfc7231.Allow,
  'rule': MULTI},
 {'_': FieldName(u'ALPN'),
  '_citations': [RFC(7639, section=(2,))],
  'iana_status': u'standard'},
 {'_': FieldName(u'Alt-Svc'),
  '_citations': [RFC(7838, section=(3,))],
  'for_request': False,
  'for_response': True,
  'iana_status': u'standard',
  'parser': rfc7838.Alt_Svc,
  'rule': SINGLE},
 {'_': FieldName(u'Alt-Used'),
  '_citations': [RFC(7838, section=(5,))],
  'bad_for_trailer': True,
  'for_request': True,
  'for_response': False,
  'iana_status': u'standard',
  'parser': rfc7838.Alt_Used,
  'precondition': False,
  'proactive_conneg': False,
  'rule': SINGLE},
 {'_': FieldName(u'Alternates'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Apply-To-Redirect-Ref'), '_citations': [RFC(4437)]},
 {'_': FieldName(u'Authentication-Info'),
  '_citations': [RFC(7615, section=(3,))],
  'iana_status': u'standard'},
 {'_': FieldName(u'Authorization'),
  '_citations': [RFC(7235, section=(4, 2))],
  'bad_for_connection': True,
  'bad_for_trailer': True,
  'for_request': True,
  'for_response': False,
  'iana_status': u'standard',
  'parser': rfc7235.Authorization,
  'precondition': False,
  'proactive_conneg': False,
  'rule': SINGLE},
 {'_': FieldName(u'C-Ext'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'C-Man'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'C-Opt'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'C-PEP'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'C-PEP-Info'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Cache-Control'),
  '_citations': [RFC(7234, section=(5, 2))],
  'bad_for_connection': True,
  'bad_for_trailer': True,
  'for_request': True,
  'for_response': True,
  'iana_status': u'standard',
  'parser': rfc7234.Cache_Control,
  'precondition': False,
  'proactive_conneg': False,
  'rule': MULTI},
 {'_': FieldName(u'CalDAV-Timezones'),
  '_citations': [RFC(7809, section=(7, 1))],
  'iana_status': u'standard'},
 {'_': FieldName(u'Close'),
  '_citations': [RFC(7230, section=(8, 1))],
  'iana_status': 'reserved'},
 {'_': FieldName(u'Compliance'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Connection'),
  '_citations': [RFC(7230, section=(6, 1))],
  'for_request': True,
  'for_response': True,
  'iana_status': u'standard',
  'parser': rfc7230.Connection,
  'precondition': False,
  'proactive_conneg': False,
  'rule': MULTI},
 {'_': FieldName(u'Content-Base'),
  '_citations': [RFC(2068), RFC(2616)],
  'iana_status': u'obsoleted'},
 {'_': FieldName(u'Content-Disposition'),
  '_citations': [RFC(6266, section=(4,))],
  'for_request': False,
  'for_response': True,
  'iana_status': u'standard',
  'parser': rfc6266.content_disposition,
  'representation_metadata': True,
  'rule': SINGLE},
 {'_': FieldName(u'Content-Encoding'),
  '_citations': [RFC(7231, section=(3, 1, 2, 2))],
  'bad_for_connection': True,
  'bad_for_trailer': True,
  'iana_status': u'standard',
  'parser': rfc7231.Content_Encoding,
  'precondition': False,
  'proactive_conneg': False,
  'representation_metadata': True,
  'rule': MULTI},
 {'_': FieldName(u'Content-ID'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Content-Language'),
  '_citations': [RFC(7231, section=(3, 1, 3, 2))],
  'bad_for_connection': True,
  'iana_status': u'standard',
  'parser': rfc7231.Content_Language,
  'precondition': False,
  'proactive_conneg': False,
  'representation_metadata': True,
  'rule': MULTI},
 {'_': FieldName(u'Content-Length'),
  '_citations': [RFC(7230, section=(3, 3, 2))],
  'bad_for_trailer': True,
  'for_request': True,
  'for_response': True,
  'iana_status': u'standard',
  'parser': rfc7230.Content_Length,
  'precondition': False,
  'proactive_conneg': False,
  'rule': SINGLE},
 {'_': FieldName(u'Content-Location'),
  '_citations': [RFC(7231, section=(3, 1, 4, 2))],
  'iana_status': u'standard',
  'parser': rfc7231.Content_Location,
  'precondition': False,
  'proactive_conneg': False,
  'representation_metadata': True,
  'rule': SINGLE},
 {'_': FieldName(u'Content-MD5'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Content-Range'),
  '_citations': [RFC(7233, section=(4, 2))],
  'bad_for_connection': True,
  'bad_for_trailer': True,
  'for_request': False,
  'for_response': True,
  'iana_status': u'standard',
  'parser': rfc7233.Content_Range,
  'rule': SINGLE},
 {'_': FieldName(u'Content-Script-Type'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Content-Style-Type'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Content-Transfer-Encoding'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Content-Type'),
  '_citations': [RFC(7231, section=(3, 1, 1, 5))],
  'bad_for_trailer': True,
  'iana_status': u'standard',
  'parser': rfc7231.Content_Type,
  'precondition': False,
  'proactive_conneg': False,
  'representation_metadata': True,
  'rule': SINGLE},
 {'_': FieldName(u'Content-Version'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Cookie'),
  '_citations': [RFC(6265)],
  'iana_status': u'standard',
  'precondition': False,
  'proactive_conneg': False},
 {'_': FieldName(u'Cookie2'),
  '_citations': [RFC(2965), RFC(6265)],
  'iana_status': u'obsoleted'},
 {'_': FieldName(u'Cost'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'DASL'),
  '_citations': [RFC(5323)],
  'iana_status': u'standard'},
 {'_': FieldName(u'DAV'),
  '_citations': [RFC(4918)],
  'iana_status': u'standard'},
 {'_': FieldName(u'Date'),
  '_citations': [RFC(7231, section=(7, 1, 1, 2))],
  'bad_for_trailer': True,
  'for_request': True,
  'for_response': True,
  'iana_status': u'standard',
  'precondition': False,
  'proactive_conneg': False,
  'parser': rfc7231.Date,
  'rule': SINGLE},
 {'_': FieldName(u'Default-Style'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Delta-Base'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Depth'),
  '_citations': [RFC(4918)],
  'iana_status': u'standard'},
 {'_': FieldName(u'Derived-From'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Destination'),
  '_citations': [RFC(4918)],
  'iana_status': u'standard'},
 {'_': FieldName(u'Differential-ID'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Digest'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'EDIINT-Features'), '_citations': [RFC(6017)]},
 {'_': FieldName(u'ETag'),
  '_citations': [RFC(7232, section=(2, 3))],
  'bad_for_connection': True,
  'for_request': False,
  'for_response': True,
  'iana_status': u'standard',
  'parser': rfc7232.ETag,
  'representation_metadata': True,
  'rule': SINGLE},
 {'_': FieldName(u'Expect'),
  '_citations': [RFC(7231, section=(5, 1, 1))],
  'bad_for_trailer': True,
  'for_request': True,
  'for_response': False,
  'iana_status': u'standard',
  'parser': rfc7231.Expect,
  'precondition': False,
  'proactive_conneg': False,
  'rule': SINGLE},
 {'_': FieldName(u'Expires'),
  '_citations': [RFC(7234, section=(5, 3))],
  'bad_for_connection': True,
  'bad_for_trailer': True,
  'for_request': False,
  'for_response': True,
  'iana_status': u'standard',
  'parser': rfc7234.Expires,
  'rule': SINGLE},
 {'_': FieldName(u'Ext'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Forwarded'),
  '_citations': [RFC(7239)],
  'iana_status': u'standard'},
 {'_': FieldName(u'From'),
  '_citations': [RFC(7231, section=(5, 5, 1))],
  'for_request': True,
  'for_response': False,
  'iana_status': u'standard',
  # I'm not parsing the full RFC 5322 ``mailbox``, not right now.
  'parser': None,
  'precondition': False,
  'proactive_conneg': False,
  'rule': SINGLE},
 {'_': FieldName(u'GetProfile'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'HTTPolice-Silence'),
  'for_request': True,
  'for_response': True,
  'parser': internal.HTTPolice_Silence,
  'precondition': False,
  'proactive_conneg': False,
  'rule': MULTI},
 {'_': FieldName(u'Hobareg'),
  '_citations': [RFC(7486, section=(6, 1, 1))],
  'iana_status': 'experimental'},
 {'_': FieldName(u'Host'),
  '_citations': [RFC(7230, section=(5, 4))],
  'bad_for_trailer': True,
  'for_request': True,
  'for_response': False,
  'iana_status': u'standard',
  'parser': rfc7230.Host,
  'precondition': False,
  'proactive_conneg': False,
  'rule': SINGLE},
 {'_': FieldName(u'HTTP2-Settings'),
  '_citations': [RFC(7540, section=(3, 2, 1))],
  'for_request': True,
  'for_response': False,
  'iana_status': u'standard',
  'parser': rfc7540.HTTP2_Settings,
  'precondition': False,
  'proactive_conneg': False,
  'rule': SINGLE},
 {'_': FieldName(u'IM'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'If'),
  '_citations': [RFC(4918, section=(10, 4))],
  'bad_for_trailer': True,
  'iana_status': u'standard',
  'precondition': True},
 {'_': FieldName(u'If-Match'),
  '_citations': [RFC(7232, section=(3, 1))],
  'bad_for_trailer': True,
  'for_request': True,
  'for_response': False,
  'iana_status': u'standard',
  'parser': rfc7232.If_Match,
  'precondition': True,
  'proactive_conneg': False,
  'rule': SINGLE},
 {'_': FieldName(u'If-Modified-Since'),
  '_citations': [RFC(7232, section=(3, 3))],
  'bad_for_trailer': True,
  'for_request': True,
  'for_response': False,
  'iana_status': u'standard',
  'parser': rfc7232.If_Modified_Since,
  'precondition': True,
  'proactive_conneg': False,
  'rule': SINGLE},
 {'_': FieldName(u'If-None-Match'),
  '_citations': [RFC(7232, section=(3, 2))],
  'bad_for_trailer': True,
  'for_request': True,
  'for_response': False,
  'iana_status': u'standard',
  'parser': rfc7232.If_None_Match,
  'precondition': True,
  'proactive_conneg': False,
  'rule': SINGLE},
 {'_': FieldName(u'If-Range'),
  '_citations': [RFC(7233, section=(3, 2))],
  'bad_for_trailer': True,
  'for_request': True,
  'for_response': False,
  'iana_status': u'standard',
  'parser': rfc7233.If_Range,
  'precondition': False,
  'proactive_conneg': False,
  'rule': SINGLE},
 {'_': FieldName(u'If-Schedule-Tag-Match'),
  'bad_for_trailer': True,
  '_citations': [RFC(6638, section=(8, 3))],
  'iana_status': u'standard',
  'precondition': True},
 {'_': FieldName(u'If-Unmodified-Since'),
  '_citations': [RFC(7232, section=(3, 4))],
  'bad_for_trailer': True,
  'for_request': True,
  'for_response': False,
  'iana_status': u'standard',
  'parser': rfc7232.If_Unmodified_Since,
  'precondition': True,
  'proactive_conneg': False,
  'rule': SINGLE},
 {'_': FieldName(u'Keep-Alive'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Label'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Last-Modified'),
  '_citations': [RFC(7232, section=(2, 2))],
  'bad_for_connection': True,
  'for_request': False,
  'for_response': True,
  'iana_status': u'standard',
  'parser': rfc7232.Last_Modified,
  'representation_metadata': True,
  'rule': SINGLE},
 {'_': FieldName(u'Link'),
  '_citations': [RFC(5988, section=(5,))],
  # Not sure if this makes any sense on requests.
  # With the ``anchor`` parameter, perhaps.
  # Leaving undefined for now.
  'for_request': None,
  'for_response': True,
  'parser': rfc5988.Link,
  'precondition': False,
  'proactive_conneg': False,
  'rule': MULTI},
 {'_': FieldName(u'Location'),
  '_citations': [RFC(7231, section=(7, 1, 2))],
  'bad_for_trailer': True,
  'for_request': False,
  'for_response': True,
  'iana_status': u'standard',
  'parser': rfc7231.Location,
  'rule': SINGLE},
 {'_': FieldName(u'Lock-Token'),
  '_citations': [RFC(4918)],
  'iana_status': u'standard'},
 {'_': FieldName(u'Man'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Max-Forwards'),
  '_citations': [RFC(7231, section=(5, 1, 2))],
  'bad_for_trailer': True,
  'for_request': True,
  'for_response': False,
  'iana_status': u'standard',
  'parser': rfc7231.Max_Forwards,
  'precondition': False,
  'proactive_conneg': False,
  'rule': SINGLE},
 {'_': FieldName(u'Memento-Datetime'),
  '_citations': [RFC(7089)],
  'iana_status': 'informational'},
 {'_': FieldName(u'Message-ID'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Meter'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Method-Check'),
  '_citations': [Citation(u'W3C Web Application Formats Working Group',
                          u'http://www.w3.org/2006/appformats/')],
  'iana_status': u'deprecated'},
 {'_': FieldName(u'Method-Check-Expires'),
  '_citations': [Citation(u'W3C Web Application Formats Working Group',
                          u'http://www.w3.org/2006/appformats/')],
  'iana_status': u'deprecated'},
 {'_': FieldName(u'MIME-Version'),
  '_citations': [RFC(7231, appendix=('A', 1))],
  'iana_status': u'standard',
  'precondition': False,
  'proactive_conneg': False},
 {'_': FieldName(u'Negotiate'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Non-Compliance'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Opt'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Optional'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Ordering-Type'),
  '_citations': [RFC(4229)],
  'iana_status': u'standard'},
 {'_': FieldName(u'Origin'),
  '_citations': [RFC(6454)],
  'iana_status': u'standard',
  'precondition': False,
  'proactive_conneg': False},
 {'_': FieldName(u'Overwrite'),
  '_citations': [RFC(4918)],
  'iana_status': u'standard',
  'precondition': True},
 {'_': FieldName(u'P3P'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'PEP'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'PICS-Label'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Pep-Info'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Position'),
  '_citations': [RFC(4229)],
  'iana_status': u'standard'},
 {'_': FieldName(u'Pragma'),
  '_citations': [RFC(7234, section=(5, 4))],
  'bad_for_trailer': True,
  'for_request': True,
  'for_response': True,
  'iana_status': u'standard',
  'parser': rfc7234.Pragma,
  'precondition': False,
  'proactive_conneg': False,
  'rule': MULTI},
 {'_': FieldName(u'Prefer'),
  '_citations': [RFC(7240)],
  'iana_status': u'standard'},
 {'_': FieldName(u'Preference-Applied'),
  '_citations': [RFC(7240)],
  'iana_status': u'standard'},
 {'_': FieldName(u'ProfileObject'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Protocol'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Protocol-Info'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Protocol-Query'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Protocol-Request'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Proxy-Authenticate'),
  '_citations': [RFC(7235, section=(4, 3))],
  'for_request': False,
  'for_response': True,
  'iana_status': u'standard',
  'parser': rfc7235.Proxy_Authenticate,
  'rule': MULTI},
 {'_': FieldName(u'Proxy-Authentication-Info'),
  '_citations': [RFC(7615, section=(4,))],
  'iana_status': u'standard'},
 {'_': FieldName(u'Proxy-Authorization'),
  '_citations': [RFC(7235, section=(4, 4))],
  'bad_for_trailer': True,
  'for_request': True,
  'for_response': False,
  'iana_status': u'standard',
  'parser': rfc7235.Proxy_Authorization,
  'precondition': False,
  'proactive_conneg': False,
  'rule': SINGLE},
 {'_': FieldName(u'Proxy-Features'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Proxy-Instruction'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Public'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Public-Key-Pins'),
  '_citations': [RFC(7469)],
  'iana_status': u'standard'},
 {'_': FieldName(u'Public-Key-Pins-Report-Only'),
  '_citations': [RFC(7469)],
  'iana_status': u'standard'},
 {'_': FieldName(u'Range'),
  '_citations': [RFC(7233, section=(3, 1))],
  'bad_for_trailer': True,
  'for_request': True,
  'for_response': False,
  'iana_status': u'standard',
  'parser': rfc7233.Range,
  'precondition': False,
  'proactive_conneg': False,
  'rule': SINGLE},
 {'_': FieldName(u'Redirect-Ref'), '_citations': [RFC(4437)]},
 {'_': FieldName(u'Referer'),
  '_citations': [RFC(7231, section=(5, 5, 2))],
  'for_request': True,
  'for_response': False,
  'iana_status': u'standard',
  'parser': rfc7231.Referer,
  'precondition': False,
  'proactive_conneg': False,
  'rule': SINGLE},
 {'_': FieldName(u'Referer-Root'),
  '_citations': [Citation(u'W3C Web Application Formats Working Group',
                          u'http://www.w3.org/2006/appformats/')],
  'iana_status': u'deprecated'},
 {'_': FieldName(u'Resolution-Hint'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Resolver-Location'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Retry-After'),
  '_citations': [RFC(7231, section=(7, 1, 3))],
  'bad_for_trailer': True,
  'for_request': False,
  'for_response': True,
  'iana_status': u'standard',
  'parser': rfc7231.Retry_After,
  'rule': SINGLE},
 {'_': FieldName(u'Safe'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Schedule-Reply'),
  '_citations': [RFC(6638)],
  'iana_status': u'standard'},
 {'_': FieldName(u'Schedule-Tag'),
  '_citations': [RFC(6638)],
  'iana_status': u'standard'},
 {'_': FieldName(u'Sec-WebSocket-Accept'),
  '_citations': [RFC(6455)],
  'iana_status': u'standard'},
 {'_': FieldName(u'Sec-WebSocket-Extensions'),
  '_citations': [RFC(6455)],
  'iana_status': u'standard'},
 {'_': FieldName(u'Sec-WebSocket-Key'),
  '_citations': [RFC(6455)],
  'iana_status': u'standard'},
 {'_': FieldName(u'Sec-WebSocket-Protocol'),
  '_citations': [RFC(6455)],
  'iana_status': u'standard'},
 {'_': FieldName(u'Sec-WebSocket-Version'),
  '_citations': [RFC(6455)],
  'iana_status': u'standard'},
 {'_': FieldName(u'Security-Scheme'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Server'),
  '_citations': [RFC(7231, section=(7, 4, 2))],
  'for_request': False,
  'for_response': True,
  'iana_status': u'standard',
  'parser': rfc7231.Server,
  'rule': SINGLE},
 {'_': FieldName(u'Set-Cookie'),
  '_citations': [RFC(6265)],
  'iana_status': u'standard',
  'rule': SET_COOKIE},
 {'_': FieldName(u'Set-Cookie2'),
  '_citations': [RFC(2965), RFC(6265)],
  'iana_status': u'obsoleted'},
 {'_': FieldName(u'SetProfile'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'SLUG'),
  '_citations': [RFC(5023)],
  'iana_status': u'standard'},
 {'_': FieldName(u'SoapAction'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Status-URI'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Strict-Transport-Security'),
  '_citations': [RFC(6797, section=(6, 1))],
  'for_request': False,
  'for_response': True,
  'iana_status': u'standard',
  'parser': rfc6797.Strict_Transport_Security,
  'rule': SINGLE_LIST},
 {'_': FieldName(u'SubOK'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Subst'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Surrogate-Capability'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Surrogate-Control'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'TCN'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'TE'),
  '_citations': [RFC(7230, section=(4, 3))],
  'bad_for_trailer': True,
  'for_request': True,
  'for_response': False,
  'iana_status': u'standard',
  'parser': rfc7230.TE,
  'precondition': False,
  'proactive_conneg': False,
  'rule': MULTI},
 {'_': FieldName(u'Timeout'),
  '_citations': [RFC(4918)],
  'iana_status': u'standard'},
 {'_': FieldName(u'Title'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Trailer'),
  '_citations': [RFC(7230, section=(4, 4))],
  'bad_for_trailer': True,
  'iana_status': u'standard',
  'parser': rfc7230.Trailer,
  'precondition': False,
  'proactive_conneg': False,
  'rule': MULTI},
 {'_': FieldName(u'Transfer-Encoding'),
  '_citations': [RFC(7230, section=(3, 3, 1))],
  'bad_for_trailer': True,
  'for_request': True,
  'for_response': True,
  'iana_status': u'standard',
  'parser': rfc7230.Transfer_Encoding,
  'precondition': False,
  'proactive_conneg': False,
  'rule': MULTI},
 {'_': FieldName(u'UA-Color'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'UA-Media'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'UA-Pixels'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'UA-Resolution'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'UA-Windowpixels'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'URI'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Upgrade'),
  '_citations': [RFC(7230, section=(6, 7))],
  'for_request': True,
  'for_response': True,
  'iana_status': u'standard',
  'parser': rfc7230.Upgrade,
  'precondition': False,
  'proactive_conneg': False,
  'rule': MULTI},
 {'_': FieldName(u'User-Agent'),
  '_citations': [RFC(7231, section=(5, 5, 3))],
  'for_request': True,
  'for_response': False,
  'iana_status': u'standard',
  'parser': rfc7231.User_Agent,
  'precondition': False,
  'proactive_conneg': False,
  'rule': SINGLE},
 {'_': FieldName(u'Variant-Vary'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Vary'),
  '_citations': [RFC(7231, section=(7, 1, 4))],
  'bad_for_connection': True,
  'bad_for_trailer': True,
  'for_request': False,
  'for_response': True,
  'iana_status': u'standard',
  'parser': rfc7231.Vary,
  'rule': SINGLE},
 {'_': FieldName(u'Version'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Via'),
  '_citations': [RFC(7230, section=(5, 7, 1))],
  'bad_for_connection': True,
  'for_request': True,
  'for_response': True,
  'iana_status': u'standard',
  'parser': rfc7230.Via,
  'precondition': False,
  'proactive_conneg': False,
  'rule': MULTI},
 {'_': FieldName(u'WWW-Authenticate'),
  '_citations': [RFC(7235, section=(4, 1))],
  'bad_for_connection': True,
  'for_request': False,
  'for_response': True,
  'iana_status': u'standard',
  'parser': rfc7235.WWW_Authenticate,
  'rule': MULTI},
 {'_': FieldName(u'Want-Digest'), '_citations': [RFC(4229)]},
 {'_': FieldName(u'Warning'),
  '_citations': [RFC(7234, section=(5, 5))],
  'bad_for_connection': True,
  'bad_for_trailer': True,
  'for_request': True,
  'for_response': True,
  'iana_status': u'standard',
  'parser': rfc7234.Warning_,
  'precondition': False,
  'proactive_conneg': False,
  'rule': MULTI},
 # Any common "X-" prefixed headers should be included here,
 # even if there are no citations for them.
 # This is necessary for notice 1277 to work properly.
 # Many of these are taken from Wikipedia:
 # https://en.wikipedia.org/wiki/List_of_HTTP_header_fields
 {'_': FieldName(u'X-ATT-DeviceId')},
 {'_': FieldName(u'X-Content-Duration')},
 {'_': FieldName(u'X-Content-Security-Policy')},
 {'_': FieldName(u'X-Content-Type-Options')},
 {'_': FieldName(u'X-CSRF-Token')},
 {'_': FieldName(u'X-CSRFToken')},
 {'_': FieldName(u'X-Device-Accept'),
  '_citations': [Citation(u'W3C Mobile Web Best Practices Working Group',
                          u'http://www.w3.org/2005/MWI/BPWG/')]},
 {'_': FieldName(u'X-Device-Accept-Charset'),
  '_citations': [Citation(u'W3C Mobile Web Best Practices Working Group',
                          u'http://www.w3.org/2005/MWI/BPWG/')]},
 {'_': FieldName(u'X-Device-Accept-Encoding'),
  '_citations': [Citation(u'W3C Mobile Web Best Practices Working Group',
                          u'http://www.w3.org/2005/MWI/BPWG/')]},
 {'_': FieldName(u'X-Device-Accept-Language'),
  '_citations': [Citation(u'W3C Mobile Web Best Practices Working Group',
                          u'http://www.w3.org/2005/MWI/BPWG/')]},
 {'_': FieldName(u'X-Device-User-Agent'),
  '_citations': [Citation(u'W3C Mobile Web Best Practices Working Group',
                          u'http://www.w3.org/2005/MWI/BPWG/')]},
 {'_': FieldName(u'X-Firefox-SPDY')},
 {'_': FieldName(u'X-Forwarded-For')},
 {'_': FieldName(u'X-Forwarded-Host')},
 {'_': FieldName(u'X-Forwarded-Proto')},
 {'_': FieldName(u'X-Frame-Options'),
  '_citations': [RFC(7034)],
  'iana_status': 'informational'},
 {'_': FieldName(u'X-HTTP-Method-Override')},
 {'_': FieldName(u'X-Powered-By')},
 {'_': FieldName(u'X-Requested-With')},
 {'_': FieldName(u'X-UA-Compatible')},
 {'_': FieldName(u'X-UIDH')},
 {'_': FieldName(u'X-Wap-Profile')},
 {'_': FieldName(u'X-WebKit-CSP')},
 {'_': FieldName(u'X-XSRF-Token')},
 {'_': FieldName(u'X-XSS-Protection')},
 ],
 extra_info=['bad_for_connection', 'bad_for_trailer',
             'for_request', 'for_response', 'iana_status',
             'parser', 'precondition', 'proactive_conneg',
             'representation_metadata', 'rule']
)
